/**
 * Alpus Elementor Addon JS Library
 * 
 * @author     AlpusTheme
 * @package    Alpus Elementor Addon
 * @subpackage Addon
 * @since      1.0
 * @version    1.0
 */
'use strict';

if ( 'undefined' !== typeof alpha_vars ) {

    window.theme || ( window.theme = {} );

    ( function ( $ ) {
        /**
         * jQuery Window Object
         * 
         * @var {jQuery} $window
         * @since 1.0
         */
        theme.$window = $( window );

        /**
         * jQuery Body Object
         * 
         * @var {jQuery} $body
         * @since 1.0
         */
        theme.$body = $( document.body );

        /**
         * Status
         * 
         * @var {string} status
         * @since 1.0
         */
        theme.status = 'loading';

        /**
         * Hash
         * 
         * @var {string} hash
         * @since 1.0
         */
        theme.hash = location.hash.indexOf( '&' ) > 0 ? location.hash.substring( 0, location.hash.indexOf( '&' ) ) : location.hash;

        /**
         * Detect Internet Explorer
         * 
         * @var {boolean} isIE
         * @since 1.0
         */
        theme.isIE = navigator.userAgent.indexOf( "Trident" ) >= 0;

        /**
         * Detect Edge
         * 
         * @var {boolean} isEdge
         * @since 1.0
         */
        theme.isEdge = navigator.userAgent.indexOf( "Edge" ) >= 0;

        /**
         * Detect Mobile
         * 
         * @var {boolean} isMobile
         * @since 1.0
         */
        theme.isMobile = /Android|webOS|iPhone|iPad|iPod|BlackBerry|IEMobile|Opera Mini/i.test( navigator.userAgent );

        /**
         * Get jQuery object
         * 
         * @since 1.0
         * @param {string|jQuery} selector	Selector to find
         * @param {string|jQuery} find		Find from selector root
         * @return {jQuery|Object}			jQuery Object or {each: $.noop}
         */
        theme.$ = function ( selector, find ) {
            if ( typeof selector == 'string' && typeof find == 'string' ) {
                return $( selector + ' ' + find );
            }
            if ( selector instanceof jQuery ) {
                if ( selector.is( find ) ) {
                    return selector;
                }
                if ( typeof find == 'undefined' ) {
                    return selector;
                }
                return selector.find( find );
            }
            if ( typeof selector == 'undefined' || !selector ) {
                return $( find );
            }
            if ( typeof find == 'undefined' ) {
                return $( selector );
            }
            return $( selector ).find( find );
        }

        /**
         * Request timeout by using requestAnimationFrame
         * 
         * @since 1.0
         * @param {function} fn
         * @param {number} delay
         * @return {Object} handle
         */
        theme.requestTimeout = function ( fn, delay ) {
            var handler = window.requestAnimationFrame || window.webkitRequestAnimationFrame || window.mozRequestAnimationFrame;
            if ( !handler ) {
                return setTimeout( fn, delay );
            }
            delay || ( delay = 0 );
            var start, rt = new Object();

            function loop( timestamp ) {
                if ( !start ) {
                    start = timestamp;
                }
                var progress = timestamp - start;
                progress >= delay ? fn() : rt.val = handler( loop );
            };

            rt.val = handler( loop );
            return rt;
        }

        /**
         * Request frame by using requestAnimationFrame
         * 
         * @since 1.0
         * @param {function} fn
         * @return {Object} handle
         */
        theme.requestFrame = function ( fn ) {
            return { val: ( window.requestAnimationFrame || window.webkitRequestAnimationFrame || window.mozRequestAnimationFrame )( fn ) };
        }


        /**
         * Parse options string to object
         * 
         * @since 1.0
         * @param {string} options	Options string
         * @return {object}
         */
        theme.parseOptions = function ( options ) {
            return 'string' == typeof options ? JSON.parse( options.replace( /'/g, '"' ).replace( ';', '' ) ) : {};
        }

        /**
         * Show loading overlay
         * 
         * @since 1.0
         * @param {string|jQuery} selector 
         * @param {string} type
         * @return {void}
         */
        theme.doLoading = function ( selector, type ) {
            var $selector = theme.$( selector );

            if ( $selector.find( '.d-loading' ).length ) {
                return;
            }

            if ( typeof type == 'undefined' ) {
                $selector.append( '<div class="d-loading"><i></i></div>' );
            } else if ( type == 'small' ) {
                $selector.append( '<div class="d-loading small"><i></i></div>' );
            } else if ( type == 'simple' ) {
                $selector.append( '<div class="d-loading small"></div>' );
            }

            $selector.each( function () {
                var $this = $( this );
                if ( 'static' == $this.css( 'position' ) ) {
                    $this.css( 'position', 'relative' );
                }
            } );
        }

        /**
         * Hide loading overlay
         * 
         * @since 1.0
         * @param {string|jQuery} selector
         * @return {void}
         */
        theme.endLoading = function ( selector ) {
            theme.$( selector ).find( '.d-loading' ).remove();
            theme.$( selector ).css( 'position', '' );
        }

        /**
         * Run appear animation
         * 
         * @since 1.0
         * @param {HTMLElement} el DOM Element to appear
         * @param {function} fn    Callback function
         * @param {object} intObsOptions Options
         * @return {void}
         */
        theme.appear = function ( el, fn, intObsOptions ) {

            var $this = $( el );

            if ( $this.data( 'observer-init' ) ) {
                return;
            }

            var interSectionObserverOptions = {
                rootMargin: '0px 0px 200px 0px',
                threshold: 0,
                alwaysObserve: false
            };

            if ( intObsOptions && Object.keys( intObsOptions ).length ) {
                interSectionObserverOptions = $.extend( interSectionObserverOptions, intObsOptions );
            }

            var observer = new IntersectionObserver( ( function ( entries ) {
                for ( var i = 0; i < entries.length; i++ ) {
                    var entry = entries[ i ];

                    if ( entry.intersectionRatio > 0 ) {
                        if ( typeof fn === 'string' ) {
                            var func = Function( 'return ' + functionName )();
                        } else {
                            var callback = fn;
                            callback.call( entry.target );
                            // observer.disconnect();
                        }
                        // Unobserve
                        if ( this.alwaysObserve == false ) {
                            observer.unobserve( entry.target );
                        }
                    }
                }
            } ).bind( interSectionObserverOptions ), interSectionObserverOptions );

            observer.observe( el );

            $this.data( 'observer-init', true );

            return this;
        }


        /**
         * Run appear animation
         * 
         * @since 1.0
         * @param {string} selector
         * @return {void}
         */
        theme.appearAnimate = function ( selector ) {
            var appearClass = typeof selector == 'string' && selector.indexOf( 'elementor-invisible' ) > 0 ? 'elementor-invisible' : 'appear-animate';

            theme.$( selector ).each( function () {
                var el = this;
                theme.appear( el, function () {
                    if ( el.classList.contains( appearClass ) && !el.classList.contains( 'appear-animation-visible' ) ) {
                        var settings = theme.parseOptions( el.getAttribute( 'data-settings' ) ),
                            duration = 1000;

                        if ( el.classList.contains( 'animated-slow' ) ) {
                            duration = 2000;
                        } else if ( el.classList.contains( 'animated-fast' ) ) {
                            duration = 750;
                        }

                        el.style[ 'animation-duration' ] = duration + 'ms';
                        el.style[ 'animation-delay' ] = settings._animation_delay + 'ms';
                        el.style[ 'transition-property' ] = 'visibility, opacity';
                        el.style[ 'transition-duration' ] = '0s';
                        el.style[ 'transition-delay' ] = settings._animation_delay + 'ms';

                        var animation_name = settings.animation || settings._animation || settings._animation_name;
                        animation_name && el.classList.add( animation_name );

                        el.classList.add( 'appear-animation-visible' );
                        setTimeout(
                            function () {
                                el.style[ 'transition-property' ] = '';
                                el.style[ 'transition-duration' ] = '';
                                el.style[ 'transition-delay' ] = '';

                                el.classList.add( 'animating' );

                                setTimeout( function () {
                                    el.classList.add( 'animated-done' );
                                }, duration );
                            },
                            settings._animation_delay ? settings._animation_delay + 500 : 500
                        );
                    }
                } );
            } );

            if ( typeof elementorFrontend == 'object' ) {
                theme.$window.trigger( 'resize.waypoints' );
            }
        }

        theme.scrolltoContainer = function ( $container, timeout ) {
            if ( $container.get( 0 ) ) {
                if ( window.innerWidth < 992 ) {
                    $( '.sidebar-overlay' ).trigger( 'click' );
                }
                if ( !timeout ) {
                    timeout = 600;
                }
                $( 'html, body' ).stop().animate( {
                    scrollTop: $container.offset().top - $( '.scroll-progress' ).height() - ( $( '#wpadminbar' ).length ? $( '#wpadminbar' ).outerHeight() : 0 )
                }, timeout, 'easeOutQuad' );
            }
        }

        /**
         * Initialize Sticky Content
         * 
         * @class StickyContent
         * @since 1.0
         * @param {string, Object} selector
         * @param {Object} options
         * @return {void}
         */
        theme.stickyContent = function () {
            function StickyContent( $el, options ) {
                return this.init( $el, options );
            }

            function refreshAll() {
                theme.$window.trigger( 'sticky_refresh.alpha', {
                    index: 0,
                    offsetTop: window.innerWidth > 600 && $( '#wp-toolbar' ).length && $( '#wp-toolbar' ).parent().is( ':visible' ) ? $( '#wp-toolbar' ).parent().outerHeight() : 0
                } );
            }

            function refreshAllSize( e ) {
                if ( !e || theme.windowResized( e.timeStamp ) ) {
                    theme.$window.trigger( 'sticky_refresh_size.alpha' );
                    theme.requestFrame( refreshAll );
                }
            }

            StickyContent.prototype.init = function ( $el, options ) {
                this.$el = $el;
                this.options = $.extend( true, {}, theme.defaults.sticky, options, theme.parseOptions( $el.attr( 'data-sticky-options' ) ) );
                this.scrollPos = window.pageYOffset; // issue: heavy js performance : 30.7ms
                this.originalHeight = this.$el.outerHeight();
                theme.$window
                    .on( 'sticky_refresh.alpha', this.refresh.bind( this ) )
                    .on( 'sticky_refresh_size.alpha', this.refreshSize.bind( this ) );
            }

            StickyContent.prototype.refreshSize = function ( e ) {

                var beWrap = window.innerWidth >= this.options.minWidth && window.innerWidth <= this.options.maxWidth;
                if ( typeof this.top == 'undefined' ) {
                    this.top = this.options.top;
                }

                if ( window.innerWidth >= 768 && this.getTop ) {
                    this.top = this.getTop();
                } else if ( !this.options.top ) {
                    this.top = this.isWrap ?
                        this.$el.parent().offset().top :
                        this.$el.offset().top + this.$el[ 0 ].offsetHeight;

                    // if sticky header has toggle dropdown menu, increase top
                    if ( this.$el.find( '.toggle-menu.show-home' ).length ) {
                        this.top += this.$el.find( '.toggle-menu .dropdown-box' )[ 0 ].offsetHeight;
                    }
                }

                if ( !this.isWrap ) {
                    beWrap && this.wrap();
                } else {
                    beWrap || this.unwrap();
                }
                e && theme.requestTimeout( this.refreshSize.bind( this ), 50 );

            }

            StickyContent.prototype.wrap = function () {
                this.$el.wrap( '<div class="sticky-content-wrapper"></div>' );
                this.$el.closest( '.toolbox-horizontal' ).addClass( 'horizontal-fixed' );
                this.isWrap = true;
            }

            StickyContent.prototype.unwrap = function () {
                this.$el.unwrap( '.sticky-content-wrapper' );
                this.$el.closest( '.toolbox-horizontal' ).removeClass( 'horizontal-fixed' );
                this.isWrap = false;
            }

            StickyContent.prototype.refresh = function ( e, data ) {
                var pageYOffset = window.pageYOffset /* + data.offsetTop*/; // issue: heavy js performance, 6.7ms
                var $el = this.$el;

                this.refreshSize();
                $( '.fixed.fix-top' ).each( function () {
                    pageYOffset += $( this ).outerHeight();
                } );

                var needUnsticky = false;
                if ( theme.$body.hasClass( 'side-header' ) ) {
                    if ( ( theme.$body.hasClass( 'side-on-desktop' ) && window.innerWidth > 991 ) ||
                        ( theme.$body.hasClass( 'side-on-tablet' ) && window.innerWidth > 767 ) ||
                        ( theme.$body.hasClass( 'side-on-mobile' ) && window.innerWidth > 575 ) ||
                        ( !theme.$body.hasClass( 'side-on-desktop' ) && !theme.$body.hasClass( 'side-on-tablet' ) && !theme.$body.hasClass( 'side-on-mobile' ) ) ) {
                        needUnsticky = true;
                    }
                }

                // Make sticky
                if ( !needUnsticky && ( pageYOffset >= this.top + this.originalHeight ) && this.isWrap ) {

                    // calculate height
                    this.height = $el[ 0 ].offsetHeight;
                    $el.hasClass( 'sticky-content-transparent' ) || $el.hasClass( 'fixed' ) || $el.parent().css( 'height', this.height + 'px' );

                    // update sticky status
                    if ( this.options.scrollMode ) {
                        if ( ( this.scrollPos >= window.pageYOffset && $el.hasClass( 'fix-top' ) ) ||
                            this.scrollPos <= window.pageYOffset && $el.hasClass( 'fix-bottom' ) ) {
                            $el.addClass( 'fixed' );
                            this.onFixed && this.onFixed();
                        } else {
                            $el.removeClass( 'fixed' ).css( { 'margin-top': '', 'margin-bottom': '', 'z-index': '' } );
                            this.onUnfixed && this.onUnfixed();
                        }
                        this.scrollPos = window.pageYOffset;
                    } else {
                        $el.addClass( 'fixed' );
                        this.onFixed && this.onFixed();
                    }

                    // update sticky order
                    if ( $el.hasClass( 'fixed' ) && $el.hasClass( 'fix-top' ) ) {
                        // this.zIndex = this.options.max_index - data.index;
                        this.zIndex = this.options.max_index - $( '.fix-top' ).index( $el );
                        $el.css( { 'margin-top': data.offsetTop + 'px', 'z-index': this.zIndex } );
                    } else if ( $el.hasClass( 'fixed' ) && $el.hasClass( 'fix-bottom' ) ) {
                        this.zIndex = this.options.max_index - data.index;
                        $el.css( { 'margin-bottom': data.offsetBottom + 'px', 'z-index': this.zIndex } );
                    } else {
                        $el.css( { 'transition': 'opacity .5s' } );
                    }

                    // stack offset
                    if ( $el.hasClass( 'fixed' ) ) {
                        if ( $el.hasClass( 'fix-top' ) ) {
                            data.offsetTop += $el[ 0 ].offsetHeight;
                        } else if ( $el.hasClass( 'fix-bottom' ) ) {
                            data.offsetBottom += $el[ 0 ].offsetHeight;
                        }
                    }
                } else {
                    $el.parent().css( 'height', '' );
                    $el.removeClass( 'fixed' ).css( { 'margin-top': '', 'margin-bottom': '', 'z-index': '' } );
                    this.onUnfixed && this.onUnfixed();
                }
                theme.$window.trigger( 'alpha_finish_sticky' );
            }

            // theme.$window.on( 'alpha_complete', function () {
            //     window.addEventListener( 'scroll', refreshAll, { passive: true } );
            //     theme.$window.on( 'resize', refreshAllSize );
            //     setTimeout( function () {
            //         refreshAllSize();
            //     }, 1000 );
            // } )

            return function ( selector, options ) {
                theme.$( selector ).each( function () {
                    var $this = $( this );
                    $this.data( 'sticky-content' ) || $this.data( 'sticky-content', new StickyContent( $this, options ) );
                } )
            }
        }

        /**
         * Initialize advanced motions
         *
         * @since 1.0
         * @param {string} selector
         * @param {string} action
         * @return {void}
         */
        theme.initAdvancedMotions = function ( selector, action ) {
            if ( theme.isMobile ) {
                return;
            }

            if ( typeof skrollr == 'undefined' ) {
                return;
            }

            var $selectors = '';

            if ( selector ) {
                $selectors = selector;
            } else {
                $selectors = $( '[data-plugin="skrollr"], .parallax' );
            }

            $selectors.removeAttr( 'data-bottom-top data-top data-center-top data-center' ).css( {} );
            if ( $selectors.length && skrollr.get() ) {
                skrollr.get().destroy();
            }

            if ( action == 'destroy' ) {
                $selectors.removeAttr( 'data-plugin data-options' );
                return;
            }

            $selectors.each( function ( e ) {
                var $this = $( this ),
                    options = {},
                    keys = [];

                if ( $( this ).attr( 'data-options' ) ) {
                    options = JSON.parse( $( this ).attr( 'data-options' ) );
                    keys = Object.keys( options );
                }

                if ( 'object' == typeof options && ( keys = Object.keys( options ) ).length ) {
                    keys.forEach( function ( key ) {
                        $this.attr( key, options[ key ] );
                    } )
                }
            } );

            if ( $selectors.length ) {
                skrollr.init( { forceHeight: false, smoothScrolling: true } );
            }
        }

        /**
         * Setup
         */
        $( window ).on( 'load', function () {

            // Body has been loaded.
            theme.$body.addClass( 'loaded' );

            // Touch is enabled?
            $( 'html' ).addClass( 'ontouchstart' in document ? 'touchable' : 'untouchable' );

            theme.appearAnimate( '.appear-animate' );                                               // Runs appear animations
            theme.stickyContent( '.sticky-content:not(.mobile-icon-bar):not(.sticky-toolbox)' );    // Initialize sticky content
            theme.initAdvancedMotions();						                                    // Initialize scrolling widgets
        } )
    } )( jQuery );
}