<?php
/**
 * Alpha Pro Builder Header class
 *
 * @author     AlpusTheme
 * @package    Alpus Pro
 * @subpackage Pro
 * @since      1.0
 */
defined( 'ABSPATH' ) || die;

use Elementor\Controls_Manager;

class Alpha_Pro_Header_Builder extends Alpha_Base {

	/**
	 * Constructor
	 *
	 * @since 1.0
	 */
	public function __construct() {
		// Register Document Controls.
		remove_action( 'elementor/documents/register_controls', array( Alpha_Header_Builder_Extend::get_instance(), 'register_document_controls' ) );
		add_action( 'elementor/documents/register_controls', array( $this, 'register_document_controls' ) );

		// Add controls to section.
		add_action( 'elementor/element/section/section_general/before_section_end', array( $this, 'add_section_controls' ), 10, 1 );

		// Add render options.
		add_filter( 'alpha_elementor_addon_options', array( $this, 'addon_options' ), 30, 2 );
	}

	/**
	 * Add sticky effect controls to section element
	 *
	 * @since 1.0
	 */
	public function add_section_controls( $self ) {
		// Add Sticky Effects for Header Controls.
		global $post;

		if ( ! empty( $post ) && ALPHA_NAME . '_template' == get_post_type( $post ) && 'header' == get_post_meta( $post->ID, ALPHA_NAME . '_template_type', true ) ) {
			$self->add_control(
				'section_content_sticky_offset',
				array(
					'type'        => Controls_Manager::SLIDER,
					'label'       => esc_html__( 'Scroll Distance (px)', 'alpha-pro' ),
					'description' => esc_html__( 'Choose the scroll distance to enable Sticky Content', 'alpha-pro' ),
					'condition'   => array(
						'section_content_sticky' => 'top',
					),
				),
				array(
					'position' => array(
						'at' => 'after',
						'of' => 'section_content_sticky_auto',
					),
				),
			);

			$self->add_control(
				'section_content_sticky_transparent',
				array(
					'type'         => Controls_Manager::SWITCHER,
					'label'        => esc_html__( 'Transparent Header', 'alpha-pro' ),
					'description'  => esc_html__( 'This will make the header transparent and overlap the main page.', 'alpha-pro' ),
					'return_value' => 'transparent',
					'prefix_class' => 'sticky-content-',
				)
			);
		}
	}

	/**
	 * Add sticky options to sticky content
	 *
	 * @since 1.0
	 */
	public function addon_options( $options, $settings ) {
		if ( ! empty( $settings['section_content_sticky_transparent'] ) ) {
			$options['class'] .= ' sticky-content-transparent';
		}
		if ( ! empty( $settings['section_content_sticky_offset'] ) && ! empty( $settings['section_content_sticky_offset']['size'] ) ) {
			$sticky_options                 = empty( $options['data-sticky-options'] ) ? array() : json_decode( $options['data-sticky-options'] );
			$sticky_options['top']          = $settings['section_content_sticky_offset']['size'];
			$options['data-sticky-options'] = wp_json_encode( $sticky_options );
		}
		return $options;
	}

	/**
	 * Add Side Header option
	 *
	 * @since 1.0
	 */
	public function register_document_controls( $document ) {
		if ( ! $document instanceof Elementor\Core\DocumentTypes\PageBase && ! $document instanceof Elementor\Modules\Library\Documents\Page ) {
			return;
		}

		// Add Template Builder Controls
		$id = (int) $document->get_main_id();

		if ( ALPHA_NAME . '_template' == get_post_type( $id ) ) {

			if ( $id && 'header' == get_post_meta( $id, ALPHA_NAME . '_template_type', true ) ) {

				$document->start_controls_section(
					'header_settings',
					array(
						'label' => alpha_elementor_panel_heading( esc_html__( 'Header Settings', 'alpha-pro' ) ),
						'tab'   => Elementor\Controls_Manager::TAB_SETTINGS,
					)
				);

					$document->add_control(
						'header_bg',
						array(
							'label'       => esc_html__( 'Background Color', 'alpha-pro' ),
							'description' => esc_html__( 'Controls the background color of header.', 'alpha-pro' ),
							'type'        => Elementor\Controls_Manager::COLOR,
							'selectors'   => array(
								'body:not(.side-header) .header' => 'background-color: {{VALUE}};',
								'.side-header .header-area'      => 'background-color: {{VALUE}};',
							),
						)
					);
					$document->add_control(
						'header_pos',
						array(
							'label'   => esc_html__( 'Position', 'alpha-pro' ),
							'type'    => Elementor\Controls_Manager::SELECT,
							'options' => array(
								''     => esc_html__( 'Top', 'alpha-pro' ),
								'side' => esc_html__( 'Side', 'alpha-pro' ),
							),
						)
					);

					$document->add_responsive_control(
						'side_header_width',
						array(
							'label'      => esc_html__( 'Side Header Width', 'alpha-pro' ),
							'type'       => Elementor\Controls_Manager::SLIDER,
							'size_units' => array(
								'px',
								'%',
								'vw',
							),
							'range'      => array(
								'px' => array(
									'min' => 0,
									'max' => 500,
								),
							),
							'selectors'  => array(
								'.page-wrapper' => '--alpha-side-header-width: {{SIZE}}{{UNIT}};',
							),
							'condition'  => array(
								'header_pos' => 'side',
							),
						)
					);

					$document->add_control(
						'side_header_breakpoint',
						array(
							'label'     => esc_html__( 'Side Header Breakpoint', 'alpha-pro' ),
							'type'      => Elementor\Controls_Manager::SELECT,
							'default'   => '',
							'options'   => array(
								''        => esc_html__( 'Never', 'alpha-pro' ),
								'desktop' => esc_html__( 'Desktop', 'alpha-pro' ),
								'tablet'  => esc_html__( 'Tablet', 'alpha-pro' ),
								'mobile'  => esc_html__( 'Mobile', 'alpha-pro' ),
							),
							'condition' => array(
								'header_pos' => 'side',
							),
						)
					);

				$document->end_controls_section();
			}
		}
	}
}

Alpha_Pro_Header_Builder::get_instance();
